import 'package:matrix/matrix.dart';

class SpaceRoomsChunkBase {
  SpaceRoomsChunkBase({
    required this.childrenState,
    required this.roomType,
  });

  SpaceRoomsChunkBase.fromJson(Map<String, dynamic> json)
      : childrenState = (json['children_state'] as List)
            .map((v) => MatrixEvent.fromJson(v))
            .toList(),
        roomType = json['room_type'] as String;

  Map<String, dynamic> toJson() => {
        'children_state': childrenState.map((v) => v.toJson()).toList(),
        'room_type': roomType,
      };

  /// The [`m.space.child`](#mspacechild) events of the space-room, represented
  /// as [Stripped State Events](#stripped-state) with an added `origin_server_ts` key.
  ///
  /// If the room is not a space-room, this should be empty.
  List<MatrixEvent> childrenState;

  /// The `type` of room (from [`m.room.create`](https://spec.matrix.org/unstable/client-server-api/#mroomcreate)), if any.
  String? roomType;
}

class SpaceRoomsChunk implements PublicRoomsChunk, SpaceRoomsChunkBase {
  SpaceRoomsChunk({
    this.avatarUrl,
    this.canonicalAlias,
    required this.guestCanJoin,
    this.joinRule,
    this.name,
    required this.numJoinedMembers,
    required this.roomId,
    this.topic,
    required this.worldReadable,
    required this.childrenState,
    required this.roomType,
  });

  SpaceRoomsChunk.fromJson(Map<String, dynamic> json)
      : avatarUrl =
            ((v) => v != null ? Uri.parse(v) : null)(json['avatar_url']),
        canonicalAlias =
            ((v) => v != null ? v as String : null)(json['canonical_alias']),
        guestCanJoin = json['guest_can_join'] as bool,
        joinRule = ((v) => v != null ? v as String : null)(json['join_rule']),
        name = ((v) => v != null ? v as String : null)(json['name']),
        numJoinedMembers = json['num_joined_members'] as int,
        roomId = json['room_id'] as String,
        topic = ((v) => v != null ? v as String : null)(json['topic']),
        worldReadable = json['world_readable'] as bool,
        childrenState = (json['children_state'] as List)
            .map((v) => MatrixEvent.fromJson((v as Map<String, dynamic>)
              ..putIfAbsent('event_id', () => 'invalid')))
            .toList(),
        roomType = json['room_type'] as String?;

  @override
  Map<String, dynamic> toJson() {
    final avatarUrl = this.avatarUrl;
    final canonicalAlias = this.canonicalAlias;
    final joinRule = this.joinRule;
    final name = this.name;
    final topic = this.topic;
    return {
      if (avatarUrl != null) 'avatar_url': avatarUrl.toString(),
      if (canonicalAlias != null) 'canonical_alias': canonicalAlias,
      'guest_can_join': guestCanJoin,
      if (joinRule != null) 'join_rule': joinRule,
      if (name != null) 'name': name,
      'num_joined_members': numJoinedMembers,
      'room_id': roomId,
      if (topic != null) 'topic': topic,
      'world_readable': worldReadable,
      'children_state': childrenState.map((v) => v.toJson()).toList(),
      'room_type': roomType,
    };
  }

  /// The URL for the room's avatar, if one is set.
  @override
  Uri? avatarUrl;

  /// The canonical alias of the room, if any.
  @override
  String? canonicalAlias;

  /// Whether guest users may join the room and participate in it.
  /// If they can, they will be subject to ordinary power level
  /// rules like any other user.
  @override
  bool guestCanJoin;

  /// The room's join rule. When not present, the room is assumed to
  /// be `public`.
  @override
  String? joinRule;

  /// The name of the room, if any.
  @override
  String? name;

  /// The number of members joined to the room.
  @override
  int numJoinedMembers;

  /// The ID of the room.
  @override
  String roomId;

  /// The topic of the room, if any.
  @override
  String? topic;

  /// Whether the room may be viewed by guest users without joining.
  @override
  bool worldReadable;

  /// The [`m.space.child`](#mspacechild) events of the space-room, represented
  /// as [Stripped State Events](#stripped-state) with an added `origin_server_ts` key.
  ///
  /// If the room is not a space-room, this should be empty.
  @override
  List<MatrixEvent> childrenState;

  /// The `type` of room (from [`m.room.create`](https://spec.matrix.org/unstable/client-server-api/#mroomcreate)), if any.
  @override
  String? roomType;

  @override
  List<String>? aliases;
}

class GetSpaceHierarchyResponse {
  GetSpaceHierarchyResponse({
    this.nextBatch,
    required this.rooms,
  });

  GetSpaceHierarchyResponse.fromJson(Map<String, dynamic> json)
      : nextBatch = ((v) => v != null ? v as String : null)(json['next_batch']),
        rooms = (json['rooms'] as List)
            .map((v) => SpaceRoomsChunk.fromJson(v))
            .toList();

  Map<String, dynamic> toJson() {
    final nextBatch = this.nextBatch;
    return {
      if (nextBatch != null) 'next_batch': nextBatch,
      'rooms': rooms.map((v) => v.toJson()).toList(),
    };
  }

  /// A token to supply to `from` to keep paginating the responses. Not present when there are
  /// no further results.
  String? nextBatch;

  /// The rooms for the current page, with the current filters.
  List<SpaceRoomsChunk> rooms;
}
