import 'dart:async';

import 'package:flutter/material.dart';

import 'package:animations/animations.dart';
import 'package:flutter_gen/gen_l10n/l10n.dart';
import 'package:matrix/matrix.dart';

import 'package:fluffychat/msc/extension_spaces_advanced/spaces_advaced.dart';
import 'package:fluffychat/msc/extension_spaces_advanced/src/space_hierarchy_model.dart';
import 'package:fluffychat/pages/chat_list/chat_list.dart';
import 'package:fluffychat/pages/chat_list/chat_list_item.dart';
import 'package:fluffychat/pages/chat_list/spaces_bottom_bar.dart';
import 'package:fluffychat/pages/chat_list/spaces_entry.dart';
import 'package:fluffychat/pages/chat_list/stories_header.dart';
import '../../utils/stream_extension.dart';
import '../../widgets/matrix.dart';
import 'recommended_room_list_item.dart';

class ChatListViewBody extends StatefulWidget {
  final ChatListController controller;

  const ChatListViewBody(this.controller, {Key? key}) : super(key: key);

  @override
  State<ChatListViewBody> createState() => _ChatListViewBodyState();
}

class _ChatListViewBodyState extends State<ChatListViewBody> {
  // the matrix sync stream
  late StreamSubscription _subscription;

  // used to check the animation direction
  String? _lastUserId;
  String? _lastSpaceId;

  @override
  void initState() {
    _subscription = Matrix.of(context)
        .client
        .onSync
        .stream
        .where((s) => s.hasRoomUpdate)
        .rateLimit(const Duration(seconds: 1))
        .listen((d) => setState(() {}));
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final reversed = !_animationReversed();
    Widget child;
    if (widget.controller.waitForFirstSync &&
        Matrix.of(context).client.prevBatch != null) {
      final rooms = widget.controller.activeSpacesEntry.getRooms(context);

      final displayStoriesHeader =
          widget.controller.activeSpacesEntry.shouldShowStoriesHeader(context);

      final space = widget.controller.activeSpacesEntry.getSpace(context);

      Future<GetSpaceHierarchyResponse?> hierarchyFuture;
      bool skipFuture;

      // check for recommended rooms in case the active space is a [SpaceSpacesEntry]
      if (widget.controller.activeSpacesEntry is SpaceSpacesEntry &&
          space != null) {
        skipFuture = false;
        hierarchyFuture = Matrix.of(context)
            .client
            .getSpaceHierarchy(space.id, suggestedOnly: true, maxDepth: 1);
      } else {
        skipFuture = true;
        hierarchyFuture = Future(() async => null);
      }

      child = FutureBuilder<GetSpaceHierarchyResponse?>(
          key: ValueKey(Matrix.of(context).client.userID.toString() +
              widget.controller.activeSpaceId.toString() +
              widget.controller.activeSpacesEntry.runtimeType.toString()),
          future: hierarchyFuture,
          builder: (context, snapshot) {
            final client = Matrix.of(context).client;

            int recommendedCount = snapshot.hasData
                ? snapshot.data?.rooms
                        .where((element) =>
                            client.rooms.indexWhere(
                                (room) => element.roomId == room.id) ==
                            -1)
                        .length ??
                    0
                : 0;

            // adding space for separator
            if (recommendedCount != 0) recommendedCount++;

            int joinedCount = rooms.length + (displayStoriesHeader ? 1 : 0);
            if (rooms.isEmpty) joinedCount++;

            final count = joinedCount + recommendedCount + 1;

            return ListView.builder(
              controller: widget.controller.scrollController,
              // add +1 space below in order to properly scroll below the spaces bar
              itemCount: count,
              itemBuilder: (BuildContext context, int i) {
                // first render the stories header
                if (displayStoriesHeader && i == 0) {
                  return const StoriesHeader();
                }
                // the room tiles afterwards
                else if (i < joinedCount) {
                  // in case there are no joined rooms, display friendly graphics
                  if (rooms.isEmpty) {
                    return Column(
                      key: const ValueKey(null),
                      mainAxisAlignment: MainAxisAlignment.center,
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        Image.asset(
                          'assets/private_chat_wallpaper.png',
                          width: 160,
                          height: 160,
                        ),
                        Center(
                          child: Text(
                            L10n.of(context)!.startYourFirstChat,
                            textAlign: TextAlign.start,
                            style: const TextStyle(
                              color: Colors.grey,
                              fontSize: 16,
                            ),
                          ),
                        ),
                      ],
                    );
                  }
                  final room = rooms[i - (displayStoriesHeader ? 1 : 0)];
                  return ChatListItem(
                    room,
                    selected:
                        widget.controller.selectedRoomIds.contains(room.id),
                    onTap: widget.controller.selectMode == SelectMode.select
                        ? () => widget.controller.toggleSelection(room.id)
                        : null,
                    onLongPress: () =>
                        widget.controller.toggleSelection(room.id),
                    activeChat: widget.controller.activeChat == room.id,
                  );
                  // display a trailing empty list tile at last position to avoid overflow with bottom bar
                } else if (i == count - 1) {
                  return skipFuture || snapshot.hasData
                      ? const ListTile()
                      : Column(
                          children: const [
                            Center(
                              child: CircularProgressIndicator(),
                            ),
                            ListTile(),
                          ],
                        );
                  // at the last position before the recommendations, show separator
                } else if (i == joinedCount - (displayStoriesHeader ? 1 : 0)) {
                  return Column(
                    children: [
                      const ListTile(),
                      const Divider(),
                      ListTile(
                          leading: const Icon(Icons.explore),
                          title: Text(L10n.of(context)!.discoverGroups))
                    ],
                  );
                  // only recommendation tiles left
                } else {
                  final roomPreview = snapshot.data!.rooms
                      .where((element) =>
                          client.rooms.indexWhere(
                              (room) => element.roomId == room.id) ==
                          -1)
                      .toList()[i - joinedCount - 1];
                  return RecommendedRoomListItem(room: roomPreview);
                }
              },
            );
          });
    } else {
      const dummyChatCount = 5;
      final titleColor =
          Theme.of(context).textTheme.bodyText1!.color!.withAlpha(100);
      final subtitleColor =
          Theme.of(context).textTheme.bodyText1!.color!.withAlpha(50);
      child = ListView.builder(
        itemCount: dummyChatCount,
        itemBuilder: (context, i) => Opacity(
          opacity: (dummyChatCount - i) / dummyChatCount,
          child: ListTile(
            leading: CircleAvatar(
              backgroundColor: titleColor,
              child: CircularProgressIndicator(
                strokeWidth: 1,
                color: Theme.of(context).textTheme.bodyText1!.color,
              ),
            ),
            title: Row(
              children: [
                Expanded(
                  child: Container(
                    height: 14,
                    decoration: BoxDecoration(
                      color: titleColor,
                      borderRadius: BorderRadius.circular(3),
                    ),
                  ),
                ),
                const SizedBox(width: 36),
                Container(
                  height: 14,
                  width: 14,
                  decoration: BoxDecoration(
                    color: subtitleColor,
                    borderRadius: BorderRadius.circular(14),
                  ),
                ),
                const SizedBox(width: 12),
                Container(
                  height: 14,
                  width: 14,
                  decoration: BoxDecoration(
                    color: subtitleColor,
                    borderRadius: BorderRadius.circular(14),
                  ),
                ),
              ],
            ),
            subtitle: Container(
              decoration: BoxDecoration(
                color: subtitleColor,
                borderRadius: BorderRadius.circular(3),
              ),
              height: 12,
              margin: const EdgeInsets.only(right: 22),
            ),
          ),
        ),
      );
    }
    return PageTransitionSwitcher(
      reverse: reversed,
      transitionBuilder: (
        Widget child,
        Animation<double> primaryAnimation,
        Animation<double> secondaryAnimation,
      ) {
        return SharedAxisTransition(
          animation: primaryAnimation,
          secondaryAnimation: secondaryAnimation,
          transitionType:
              widget.controller.snappingSheetController.currentPosition ==
                      kSpacesBottomBarHeight
                  ? SharedAxisTransitionType.horizontal
                  : SharedAxisTransitionType.vertical,
          fillColor: Theme.of(context).scaffoldBackgroundColor,
          child: child,
        );
      },
      child: child,
    );
  }

  @override
  void dispose() {
    _subscription.cancel();
    super.dispose();
  }

  bool _animationReversed() {
    bool reversed;
    // in case the matrix id changes, check the indexOf the matrix id
    final newClient = Matrix.of(context).client;
    if (_lastUserId != newClient.userID) {
      reversed = Matrix.of(context)
              .currentBundle!
              .indexWhere((element) => element!.userID == _lastUserId) <
          Matrix.of(context)
              .currentBundle!
              .indexWhere((element) => element!.userID == newClient.userID);
    }
    // otherwise, the space changed...
    else {
      reversed = widget.controller.spaces
              .indexWhere((element) => element.id == _lastSpaceId) <
          widget.controller.spaces.indexWhere(
              (element) => element.id == widget.controller.activeSpaceId);
    }
    _lastUserId = newClient.userID;
    _lastSpaceId = widget.controller.activeSpaceId;
    return reversed;
  }
}
